"""
Constant values
"""

#: string: Name of remote git source to look for in current repository
DEFAULT_REMOTE = "origin"

#: string: Format for label if not set, and if a coderef is provided. Available vars:
#   filename    Filename component
#   coderef     Code reference
DEFAULT_LABEL_FORMAT = "{coderef}"
